# -*- coding: iso-8859-1 -*-
#
# $Id: MultipleChoiceQuestion.py,v 1.7 2005/05/20 10:11:15 mxp Exp $
#
# Copyright  2004 Otto-von-Guericke-Universitt Magdeburg
#
# This file is part of LlsMultipleChoice.
#
# LlsMultipleChoice is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LlsMultipleChoice is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LlsMultipleChoice; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import random

from AccessControl import ClassSecurityInfo

from Acquisition import aq_base, aq_acquire, aq_inner, aq_parent

from Products.Archetypes.public import BaseFolder, BaseFolderSchema, \
BaseContent, BaseSchema, Schema, BooleanField, BooleanWidget, IntegerField, IntegerWidget, StringField, \
TextField, SelectionWidget, TextAreaWidget, StringWidget, RichWidget, registerType

from Products.LlsMultipleChoice.config import PROJECTNAME, SKINS_DIR, \
    QUESTION_DIR, QUESTION_TYPES, ANSWER_DIR, \
    ANSWER_TYPES, GLOBALS, PERMISSION_INTERROGATOR, \
    PERMISSION_STUDENT, CUSTOM_EVALUATION_FUNCTION_NAME, I18N_DOMAIN
from Products.LlsMultipleChoice.tools import getParent, evalFunString, log
from Products.LlsMultipleChoice.QuestionTypes.SelectionQuestion import SelectionQuestion
from Products.LlsMultipleChoice.QuestionTypes.PointsQuestion import PointsQuestion
from Products.LlsMultipleChoice.AnswerTypes.MultipleChoiceAnswer import MultipleChoiceAnswer


class MultipleChoiceQuestion(SelectionQuestion, PointsQuestion):
    """ This class represents a question in a multiple chopice test.
        The answers can be either right or wrong. The candidate has to select
        which answer(s) he/she thinks is (are) correct.
    """

    # See comments in 'AbstractQuestionContainer' for an explanation
    # of the function of a schema, therein defined properties (fields) and
    # internationalization of the widgets
    schema = SelectionQuestion.schema + PointsQuestion.schema

    # Use a custom page template for viewing.
    actions = (
        {
            'id': 'view',
            'action': 'string:${object_url}/multiplechoicequestion_view',
            'permissions': (PERMISSION_INTERROGATOR,),
        },
    )

    # Only Answer objects may be put into this folder.
    allowed_content_types = (MultipleChoiceAnswer.archetype_name,)
    meta_type = portal_type = archetype_name = 'MC Question'

    security = ClassSecurityInfo()

    security.declarePrivate('getCorrectAnswerIds')
    def getCorrectAnswerIds(self, candidateId):
        """ Return the IDs of the correct answers to this question that
            were presented to the candidate with ID candidateId.

            @param candidateId The user ID of the candidate.
        """
        # all the answers to this question
        correctAnswers = self.listFolderContents()
        # filter out the correct ones
        correctAnswers = filter((lambda answer : answer.isCorrect()) , correctAnswers)
        # get the IDs of the answers
        correctAnswerIds = map((lambda answer : answer.getId()), correctAnswers)
        # filter out the ones that were presented to the candidate
        parent = getParent(self)
        suggestedAnswerIds = parent.getAnswers(candidateId, self.UID())
        correctSuggestedAnswerIds = filter((lambda answerId : answerId in suggestedAnswerIds) , correctAnswerIds)
        return correctSuggestedAnswerIds


    def getCandidatePoints(self, candidateId):
        """ Return how many points the user got for this question.

            @param candidateId The user ID of the candidate whose points you want to know.

            If a custom evaluation script has been uploaded it will be
            invoked. Otherwise a default method will be used.
        """
        parent = getParent(self)
        customScript = parent.getEvaluationScript(self.archetype_name)
        answer = parent.getCandidateAnswer(candidateId, self.UID())
        if(not customScript): # default
            if answer is None:
                return 0
            else:
                if(answer == None):
                    answer = []
                actualAnswerIds = self.getCorrectAnswerIds(candidateId)
                answer.sort()
                actualAnswerIds.sort()
                return [0, self.getPoints()][answer == actualAnswerIds]
        else: # use custom script
            return evalFunString(customScript, CUSTOM_EVALUATION_FUNCTION_NAME, \
                [self, candidateId, answer])


# Register this type in Zopes
try:
    registerType(MultipleChoiceQuestion)
    log('Worked: registerType(MultipleChoiceQuestion)\n')
except Exception, e:
    log('Failed: registerType(MultipleChoiceQuestion): ' + unicode(e) + '\n')
    raise e
    